/*
 * Program memory segment support.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/re/c/dataspace.h>
#include <l4/re/c/rm.h>

#include <fsclient/client.h>

#include <exec/mapped_region.h>



/* Program segment abstraction. */

class Segment
{
protected:
  MappedRegion _region;

public:
  /* Allocated memory. */

  char *buf;
  l4re_ds_t ds;

  /* Segment base and corresponding region base. */

  l4_addr_t base, region_base;

  /* Segment size and corresponding region size. */

  offset_t size, region_size;

  /* Offset of segment content within the region. */

  offset_t region_offset;

  /* Access flags. */

  l4re_rm_flags_t flags;

  /* File access details. */

  offset_t file_offset, file_contents;

  explicit Segment(offset_t base, offset_t size, l4re_rm_flags_t flags,
                   offset_t file_offset = 0, offset_t file_contents = 0);

  long allocate();

  long fill(file_t *file);

  MappedRegion &region();

  l4_addr_t region_address(char *address);

  l4_addr_t region_address(l4_addr_t address);
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
